@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.5 2007/05/17 06:46:01 yamaguti Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.5.2.3 2007/08/01 04:26:57 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -66,6 +66,7 @@ static RETSIGTYPE close_idle_connection(int sig);
 static int send_params(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
 static void send_frontend_exits(void);
 static int s_do_auth(POOL_CONNECTION_POOL_SLOT *cp, char *password);
+static int select_load_balancing_node(void);
 
 /*
  * non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived
@@ -200,6 +201,13 @@ void do_child(int unix_fd, int inet_fd)
 		idle = 0;
 		child_idle_sec = 0;
 
+		/* check backend timer is expired */
+		if (backend_timer_expired)
+		{
+			pool_backend_timer();
+			backend_timer_expired = 0;
+		}
+
 		/* disable timeout */
 		pool_disable_timeout();
 
@@ -397,6 +405,9 @@ void do_child(int unix_fd, int inet_fd)
 				 sp->user, sp->database, remote_ps_data);
 		set_ps_display(psbuf, false);
 
+		/* select load balancing node */
+		backend->info->load_balancing_node = select_load_balancing_node();
+
 		/* query process loop */
 		for (;;)
 		{
@@ -549,6 +560,7 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 {
     fd_set	readmask;
     int fds;
+	int save_errno;
 
 	SockAddr saddr;
 	int fd = 0;
@@ -561,7 +573,7 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 	static int cnt;
 #endif
 	struct timeval *timeoutval;
-	struct timeval tv1, tv2, tmback;
+	struct timeval tv1, tv2, tmback = {0, 0};
 
 	char remote_host[NI_MAXHOST];
 	char remote_port[NI_MAXSERV];
@@ -590,6 +602,14 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 
 	fds = select(Max(unix_fd, inet_fd)+1, &readmask, NULL, NULL, timeoutval);
 
+	save_errno = errno;
+	/* check backend timer is expired */
+	if (backend_timer_expired)
+	{
+		pool_backend_timer();
+		backend_timer_expired = 0;
+	}
+
 	/*
 	 * following code fragment computes remaining timeout val in a
 	 * portable way. Linux does this automazically but other platforms do not.
@@ -622,6 +642,8 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 #endif
 	}
 
+	errno = save_errno;
+
 	if (fds == -1)
 	{
 		if (errno == EAGAIN || errno == EINTR)
@@ -659,13 +681,22 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 	gettimeofday(&now1,0);
 #endif
 	afd = accept(fd, (struct sockaddr *)&saddr.addr, &saddr.salen);
+
+	save_errno = errno;
+	/* check backend timer is expired */
+	if (backend_timer_expired)
+	{
+		pool_backend_timer();
+		backend_timer_expired = 0;
+	}
+	errno = save_errno;
 	if (afd < 0)
 	{
 		/*
 		 * "Resource temporarily unavailable" (EAGAIN or EWOULDBLOCK)
-		 * can be silently ignored.
+		 * can be silently ignored. And EINTR can be ignored.
 		 */
-		if (errno != EAGAIN && errno != EWOULDBLOCK)
+		if (errno != EAGAIN && errno != EWOULDBLOCK && errno != EINTR)
 			pool_error("accept() failed. reason: %s", strerror(errno));
 		return NULL;
 	}
@@ -1610,3 +1641,41 @@ static int s_do_auth(POOL_CONNECTION_POOL_SLOT *cp, char *password)
 	}
 	return -1;
 }
+
+
+/*
+ * Select load balancing node
+ */
+static int select_load_balancing_node(void)
+{
+	double total_weight,r;
+	int i;
+
+	/* choose a backend in random manner with weight */
+	selected_slot = 0;
+	total_weight = 0.0;
+
+	for (i=0;i<NUM_BACKENDS;i++)
+	{
+		if (VALID_BACKEND(i))
+		{
+			total_weight += BACKEND_INFO(i).backend_weight;
+		}
+	}
+	r = (((double)random())/RAND_MAX) * total_weight;
+	total_weight = 0.0;
+	for (i=0;i<NUM_BACKENDS;i++)
+	{
+		if (VALID_BACKEND(i))
+		{
+			if(r >= total_weight)
+				selected_slot = i;
+			else
+				break;
+			total_weight += BACKEND_INFO(i).backend_weight;
+		}
+	}
+
+	pool_debug("select_load_balancing_node: selected backend id is %d", selected_slot);
+	return selected_slot;
+}
